#!/bin/bash

base=$(dirname $0)

set -e

mailmanlist=$1
if [ -z "$mailmanlist" ]; then
    echo "Usage: $0 LIST@lists.DOMAIN" >&2
    exit 1
fi
mmlistname=${mailmanlist%@*}
mmdomain=${mailmanlist#*@}
sympalistname=$mmlistname
robot=${mmdomain/lists/sympa}
if [ -n "$2" ]; then
    sympalistname=${2%@*}
    robot=${2#*@}
fi
mmarchive=/var/lib/mailman/archives/private/$mmlistname.mbox/$mmlistname.mbox

if [ ! -f "/etc/sympa/$robot/robot.conf" ]; then
    echo "robot $robot does not exist" >&2
    exit 1
fi

if [ -z "$MMPW" ]; then
    echo "Please set MMPW to the listadmin password of $mmdomain!" >&2
    exit 1
fi

function sshml() {
    ssh -o ControlPath="~/.ssh/cp-$MPI_ROOT-%C" -o ControlMaster=auto -o ControlPersist=600s $mmdomain -l root "$@"
}

wwsympa_url=$(awk '$1=="wwsympa_url" { print $2; }' /etc/sympa/$robot/robot.conf)

echo importing $mailmanlist as $sympalistname@$robot

tmpdir=$(mktemp -t -d mm2sympa.${mailmanlist/@/AT}.XXXXXXXX)
# sympa user needs access
chmod go+rx $tmpdir
trap "rm -rf $tmpdir" 0 1 2 5 15 EXIT

#set -x

# create list based on mailman config
inputpy=$tmpdir/input.py
sshml config_list -o /dev/stdout $mmlistname > $inputpy
# make all strings unicode
sed -i -e "s/ '/ u'/" -e 's/ "/ u"/' $inputpy
# enhance config with additional variables for gen_config.py
cat <<EOF >> $inputpy
old_listname = u'$mmlistname'
new_listname = u'$sympalistname'
robot = u'$robot'
url = u'$wwsympa_url'
EOF
configxml=$tmpdir/input.xml
infofile=$tmpdir/info
blocklist=$tmpdir/blocklist.txt
whitelist=$tmpdir/whitelist.txt
$base/gen_config.py $inputpy $configxml $infofile $blocklist $whitelist
# sympa user needs access
chmod 0644 $configxml

# fetch members
dumpfile=$tmpdir/member.dump
$base/mailman-subscribers3.py \
       --url_path /admin \
       --ssl \
       --sympa \
       $mmdomain $mmlistname "$MMPW" \
       > $dumpfile

sympa --create_list --robot $robot --input_file $configxml

# import archive
localarchive=$tmpdir/archive.mbox
if sshml cat $mmarchive > $localarchive; then
    chmod 644 $localarchive
    $base/import_mbox.sh $localarchive $sympalistname $robot > $tmpdir/archive.log
else
    echo "no archive, thus no archive import needed"
fi

listdir=/var/lib/sympa/list_data/$robot/$sympalistname
install -m 640 -o sympa -g sympa $dumpfile $listdir
sympa --restore_users --list=$sympalistname@$robot

search_filters=$listdir/search_filters
install -d -m 750 -o sympa -g sympa $search_filters
for f in  $blocklist $whitelist; do
    [ -s $f ] || continue
    install -m 640 -o sympa -g sympa $f $search_filters
done

echo sshml rmlist -a $mmlistname
